#include <sstream>
#include <inicpp.h>
#include <raylib.h>

#include "Panda.hpp"

namespace
{
  struct TextureAtlas
  {
    Texture2D texture;
    std::unordered_map<std::string, panda::TextureAtlasItem> items;
  };
}

static std::unordered_map<std::string, TextureAtlas> TextureAtlasMap;

bool panda::LoadTextureAtlas(const char *image_path, const char *ini_path, std::string AtlasKey)
{

  ini::IniFile ini;
  TextureAtlas atlas;

  char* text = LoadFileText(ini_path);
  ini.decode(text);

  // load texture
  atlas.texture = LoadTexture(image_path);

  // loop item
  std::stringstream ss(ini["meta"]["list"].as<const char*>());
  std::string item;
  while (std::getline(ss, item, ';')) {
    atlas.items.insert({item, TextureAtlasItem{
      ini[item]["x"].as<int>(),
      ini[item]["y"].as<int>(),
      ini[item]["w"].as<int>(),
      ini[item]["h"].as<int>(),
      ini[item]["px"].as<float>(),
      ini[item]["py"].as<float>(),
    }});
  }

  UnloadFileText(text);

  TextureAtlasMap.insert({AtlasKey, atlas});
  return true;
}

bool panda::UnloadTextureAtlas(std::string AtlasKey)
{
  // looks like its already clear automatically (when close window)
  // just incase if it not when changing scene
  if (TextureAtlasMap.count(AtlasKey))
  {
    TextureAtlas &atlas = TextureAtlasMap.at(AtlasKey);
    UnloadTexture(atlas.texture);
    TextureAtlasMap.erase(AtlasKey);
  }
  return true;
}

void panda::DrawTextureAtlas(std::string AtlasKey, std::string key, int x, int y)
{
  DrawTextureAtlasPro(AtlasKey, key, x, y, WHITE);
}

void panda::DrawTextureAtlasPro(std::string AtlasKey, std::string key, int x, int y, Color color, bool flipX, bool flipY)
{
  TextureAtlas &atlas = TextureAtlasMap.at(AtlasKey);
  TextureAtlasItem &item = atlas.items.at(key);

  float _flipX = (flipX) ? -1.0f : 1.0f;
  float _flipY = (flipY) ? -1.0f : 1.0f;

  // todo flip position
  DrawTextureRec(
    atlas.texture, 
    {(float)item.x, (float)item.y, item.w*_flipX, item.h*_flipY}, 
    {x - item.px*item.w, y - item.py*item.h}, color);
}
